/*
 * rctest.cc
 *
 *  Created on: 4-Mar-2009
 *      Author: ltracy
 */

#include "ns3/core-module.h"
#include "ns3/common-module.h"
#include "ns3/uan-module.h"
#include "ns3/helper-module.h"
#include "ns3/mobility-module.h"
#include "ns3/node-module.h"
#include "ns3/log.h"
#include "ns3/config.h"
#include "ns3/callback.h"

#include <sstream>
#include <iostream>
#include <cmath>
#include <fstream>

NS_LOG_COMPONENT_DEFINE("RcTest");

using namespace ns3;


uint32_t m_bytesRx;
namespace ns3
{
extern uint32_t m_cntrlSends;
}


static void
ReceivePacket(Ptr<Socket> socket)
{

  Ptr<Packet> packet;
  while(packet = socket->Recv())
    {
      m_bytesRx += packet->GetSize();
    }

}

static void
TraceCycle(std::ostream *os, Time startTime,
           Time minPdelay, uint32_t numRes, uint32_t bytes,
           double winSec, uint32_t ctlRate, double retryRate)
{
  *os << startTime.GetSeconds() << " " << minPdelay.GetSeconds() << " " << numRes << " "
      << bytes << " " << winSec << " " << ctlRate << " " << retryRate << std::endl;
}

//static void
//SendPacket(Ptr<NetDevice> dev)
//{
//  NS_LOG_DEBUG(Simulator::Now().GetSeconds() << " Sending packet");
//  Ptr<Packet> pkt = Create<Packet>(1000);
//  dev->Send(pkt, UanAddress(UanAddress::GetBroadcast()), 0);
//}

UanTxMode
CreateMode(uint32_t ktot, uint32_t kass, uint32_t b, uint32_t m, uint32_t fc, bool upperblock, std::string name)
{

  std::ostringstream buf;
  buf << name << " " << kass;

  uint32_t rate = b/ktot* (kass) * m;
  uint32_t bw = kass * b / ktot;
  uint32_t fcmode;
  if(upperblock)
    fcmode = (b - bw)/2 + fc;
  else
    fcmode = (uint32_t) ((-((double) b ) + (double) bw)/2.0 + (double) fc);


  uint32_t phyrate = b;

  UanTxMode mode;
  mode = UanTxModeFactory::CreateMode(UanTxMode::OTHER,
                                      rate,
                                      phyrate,
                                      fcmode,
                                      bw,
                                      (uint32_t) std::pow(double(2),double(m)),
                                      buf.str());
  return mode;

}

//Want rate 0 = fastest data rate/slowest control channel rate.
//Assume over simplified OFDMA with K available subcarriers, total bandwidth B and m bits per symbol
//Also assume that we can divy up between the two channels by ind. subcarrier, i.e. # of rates = K
void
CreateDualModes(UanModesList &dataModes, UanModesList &controlModes, uint32_t k, uint32_t b, uint32_t m, uint32_t fc)
{


  for(uint32_t i=1;i < k; i++)
    {
      controlModes.AppendMode(CreateMode(k, i, b, m, fc, false, "control "));
    }
  for(uint32_t i=k-1; i > 0; i--)
    {
      dataModes.AppendMode(CreateMode(k, i, b, m, fc, true, "data "));
    }
}


int
main(int argc, char *argv[])
{

  LogComponentEnable("UanMacRc", LOG_LEVEL_ALL);
  LogComponentEnable("UanMacRcGw", LOG_LEVEL_ALL);
  //LogComponentEnable("UanPhyDual", LOG_LEVEL_ALL);
  LogComponentEnable("RcTest", LOG_LEVEL_ALL);
  //LogComponentEnable("UanPhyGen", LOG_LEVEL_ALL);
  //LogComponentEnable("UanTransducerHd", LOG_LEVEL_ALL);

  uint32_t totalRate=4096;
  uint32_t numRates=1024;
  uint32_t amin = 1;
  uint32_t amax = 50;
  uint32_t runNumber=0;
  uint32_t seed=0;
  uint32_t maxr=1000;
  uint32_t N=30;
  double SIFS = 0.05;
  double simTime = 5000.0;
  uint32_t pktSize = 1000;
  std::string tputfile = "rctest.dat";
  std::string cycleFile = "cycles.dat";

  CommandLine cmd;
  cmd.AddValue("RunNumber", "RunNumber to give to RNG", runNumber);
  cmd.AddValue("TotalRate", "Total channel capacity", totalRate);
  cmd.AddValue("NumberRates", "Number of divided rates", numRates);
  cmd.AddValue("MaxRange", "Maximum range between gateway and acoustic node", maxr);
  cmd.AddValue("AMin", "Minimum constant to test", amin);
  cmd.AddValue("AMax", "Maximum constant to test", amax);
  cmd.AddValue("TputFile", "Filename of throughput log", tputfile);
  cmd.AddValue("NumberNodes", "Number of nodes", N);
  cmd.AddValue("SIFS", "SIFS duration (seconds)", SIFS);
  cmd.AddValue("PktSize", "Packet size in bytes", pktSize);
  cmd.AddValue("SimTime", "Seconds of simulation time", simTime);
  cmd.AddValue("CyclesFile", "Filename of file to log cycle stats to", cycleFile);

  cmd.Parse(argc,argv);


  UanModesList dataModes;
  UanModesList controlModes;

  CreateDualModes(dataModes, controlModes, numRates, totalRate, 1, 12000);

//  uint32_t printMode = 0;
//  std::cout << "Data: " << dataModes[printMode].GetDataRateBps() << ", " << dataModes[printMode].GetBandwidthHz() << ", " << dataModes[printMode].GetCenterFreqHz() << std::endl;
//  std::cout << "Control: " << controlModes[printMode].GetDataRateBps() << ", " << controlModes[printMode].GetBandwidthHz() << ", " << controlModes[printMode].GetCenterFreqHz() << std::endl;

  UanHelper uan;
  uan.SetPhy("ns3::UanPhyDual",
             "SupportedModesPhy1", UanModesListValue(dataModes),
             "SupportedModesPhy2", UanModesListValue(controlModes));

  std::ofstream data(tputfile.c_str());

  if(seed)
    {
      NS_ASSERT(SeedManager::CheckSeed(1));
      SeedManager::SetSeed(seed);
      SeedManager::SetRun(runNumber);
    }

  for(uint32_t km=amin;km<=amax;km++)
    {



      NS_LOG_DEBUG("Run number: " << runNumber);

      uan.SetMac("ns3::UanMacRcGw",
          "NumberOfRates", UintegerValue(dataModes.GetNModes()),
          "NumberOfNodes", UintegerValue(N),
          "MaxReservations", UintegerValue(km),
          "RetryRate", DoubleValue(1/30.0),
          "SIFS", TimeValue(Seconds(SIFS)),
          "MaxPropDelay", TimeValue(Seconds(maxr/1500.0)));


      Ptr<UanChannel> chan = CreateObject<UanChannel>();

      NodeContainer sink;
      sink.Create(1);


      NetDeviceContainer sinkDev = uan.Install(sink, chan);
      //Trace cycles
      std::ostringstream oss;
      uint32_t nodeid = sink.Get(0)->GetId();
      uint32_t devid = sinkDev.Get(0)->GetIfIndex();
      oss << "/NodeList/" << nodeid << "/DeviceList/" << devid << "/$ns3::UanNetDevice/Mac/Cycle";
      std::ofstream cycle(cycleFile.c_str());
      Config::ConnectWithoutContext(oss.str(), MakeBoundCallback(&TraceCycle, &cycle) );

      uan.SetMac("ns3::UanMacRc", "NumberOfRates", UintegerValue(dataModes.GetNModes()),
          "RetryRate", DoubleValue(1/100.0));

      NodeContainer nodes;
      nodes.Create(N);

      NetDeviceContainer devices = uan.Install(nodes, chan);

      MobilityHelper mobility;


      uint32_t depth = 70;
      UniformVariable urv(0,maxr);

      Ptr<ListPositionAllocator> pos = CreateObject<ListPositionAllocator>();
      pos->Add(Vector(maxr, maxr, depth));
      double rsum=0;;
      double minr = 2*maxr;
      for(uint32_t i=0;i<N;i++)
        {
	  UniformVariable uv(0, 2*M_PI);
          double theta = uv.GetValue();
          double newr = urv.GetValue();
          rsum += newr;
          minr = std::min(minr, newr);
          pos->Add(Vector(maxr + cos(theta)*newr, maxr + sin(theta)*newr, depth));

        }
      NS_LOG_DEBUG("Mean range from gateway: " << rsum/N << "    min. range " << minr);
      mobility.SetPositionAllocator(pos);
      mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel");
      mobility.Install(sink);
      mobility.Install(nodes);

      Ptr<MobilityModel> mp = sink.Get(0)->GetObject<MobilityModel>();
      NS_LOG_DEBUG("Sink position " << mp->GetPosition());
      NodeContainer::Iterator it = nodes.Begin();
      for(;it != nodes.End();it++)
        {
          Ptr<MobilityModel> mp = (*it)->GetObject<MobilityModel>();
          NS_LOG_DEBUG("Node position: " << mp->GetPosition());
        }

      PacketSocketHelper pktskth;
      pktskth.Install(nodes);
      pktskth.Install(sink);

      PacketSocketAddress socket;
      socket.SetSingleDevice(sinkDev.Get(0)->GetIfIndex());
      socket.SetPhysicalAddress(sinkDev.Get(0)->GetAddress());
      socket.SetProtocol(0);

      OnOffHelper app("ns3::PacketSocketFactory", Address(socket));
      app.SetAttribute("OnTime", RandomVariableValue(ConstantVariable(2*simTime)));
      app.SetAttribute("OffTime", RandomVariableValue(ConstantVariable(0)));
      app.SetAttribute("DataRate", DataRateValue(totalRate));
      app.SetAttribute("PacketSize", UintegerValue(pktSize));

      ApplicationContainer apps = app.Install(nodes);

      apps.Start(Seconds(0.5));
      apps.Stop((Seconds(simTime+0.5)));

      Ptr<Node> sinkNode = sink.Get(0);
      TypeId psfid = TypeId::LookupByName("ns3::PacketSocketFactory");
      if(sinkNode->GetObject<SocketFactory>(psfid) == 0)
        {
          Ptr<PacketSocketFactory> psf = CreateObject<PacketSocketFactory>();
          sinkNode->AggregateObject(psf);
        }
      Ptr<Socket> sinkSocket = Socket::CreateSocket(sinkNode, psfid);
      sinkSocket->Bind(socket);
      sinkSocket->SetRecvCallback(MakeCallback(ReceivePacket));

      Simulator::Stop(Seconds(simTime + 0.6));
      Simulator::Run();
      Simulator::Destroy();

      std::cout << "Received " << m_bytesRx << " bytes at sink" << std::endl;
      std::cout << "Control channel attempts " << m_cntrlSends << std::endl;
      double throughput = m_bytesRx*8.0/(5000.0*4096.0);
      std::cout << "a = " << km << "   Throughput = " << throughput << std::endl;

      data << km << " " << throughput << std::endl;
      m_bytesRx = 0;
      m_cntrlSends = 0;


    }
}
