/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 *
 *
 */

#ifndef UANHEADERCOMMON_H_
#define UANHEADERCOMMON_H_

#include "ns3/header.h"
#include "ns3/nstime.h"
#include "ns3/simulator.h"
#include "uan-address.h"

namespace ns3
{

/**
 * \class UanHeaderCommon
 *
 * Header includes 1 byte src address, 1 byte dest address,
 * and a 1 byte type field.
 */
class UanHeaderCommon : public ns3::Header
{
public:
  UanHeaderCommon();
  UanHeaderCommon(const UanAddress src, const UanAddress dest, uint8_t type);
  virtual ~UanHeaderCommon();

  static TypeId GetTypeId(void);

  void SetDest(UanAddress dest);
  void SetSrc(UanAddress src);
  void SetType(uint8_t type);

  UanAddress GetDest(void) const;
  UanAddress GetSrc(void) const;
  uint8_t GetType(void) const;



  //Inherrited methods
  virtual uint32_t GetSerializedSize (void) const;
  virtual void Serialize (Buffer::Iterator start) const;
  virtual uint32_t Deserialize (Buffer::Iterator start);
  virtual void Print (std::ostream &os) const;
  virtual TypeId GetInstanceTypeId(void) const;
private:

  UanAddress m_dest;
  UanAddress m_src;
  uint8_t m_type;


};

}

#endif /*UANHEADERCOMMON_H_*/
