/*
 * uan-header-rc.cc
 *
 *  Created on: 2-Mar-2009
 *      Author: ltracy
 */

#include "uan-header-rc.h"

#include <set>

namespace ns3
{

UanHeaderRcData::UanHeaderRcData() :
  Header(),
  m_frameNo(0),
  m_propDelay(Seconds(0))
{
  // TODO Auto-generated constructor stub

}

UanHeaderRcData::UanHeaderRcData(uint8_t frameNo, Time propDelay)
    : Header(),
      m_frameNo(frameNo),
      m_propDelay(propDelay)
{

}

UanHeaderRcData::~UanHeaderRcData()
{
  // TODO Auto-generated destructor stub
}


TypeId
UanHeaderRcData::GetTypeId()
{
  static TypeId tid = TypeId("ns3::UanHeaderRcData")
    .SetParent<Header> ()
    .AddConstructor<UanHeaderRcData>()
    ;
  return tid;
}

void
UanHeaderRcData::SetFrameNo(uint8_t no)
{
  m_frameNo = no;
}

void
UanHeaderRcData::SetPropDelay(Time propDelay)
{
  m_propDelay = propDelay;
}

uint8_t
UanHeaderRcData::GetFrameNo(void)
{
  return m_frameNo;
}

Time
UanHeaderRcData::GetPropDelay(void)
{
  return m_propDelay;
}

uint32_t
UanHeaderRcData::GetSerializedSize (void) const
{
  return 1 + 2;
}

void
UanHeaderRcData::Serialize (Buffer::Iterator start) const
{
  start.WriteU8(m_frameNo);
  start.WriteU16( (uint16_t) (1000.0*m_propDelay.GetSeconds() + 0.5));
}
uint32_t
UanHeaderRcData::Deserialize (Buffer::Iterator start)
{
  Buffer::Iterator rbuf = start;

  m_frameNo = start.ReadU8();
  m_propDelay = Seconds( ((double) start.ReadU16())/1000.0 );

  return rbuf.GetDistanceFrom(start);
}

void
UanHeaderRcData::Print (std::ostream &os) const
{
  os << "Frame No=" << (uint32_t) m_frameNo << " Prop Delay=" << m_propDelay.GetSeconds();
}

TypeId
UanHeaderRcData::GetInstanceTypeId(void) const
{
  return GetTypeId();
}


UanHeaderRcRts::UanHeaderRcRts() :
  Header(),
  m_frameNo(0),
  m_noFrames(0),
  m_length(0),
  m_timeStamp(Seconds(0)),
  m_retryNo(0)
{

}

UanHeaderRcRts::UanHeaderRcRts(uint8_t frameNo, uint8_t retryNo, uint8_t noFrames, uint16_t length, Time timeStamp) :
  Header(),
  m_frameNo(frameNo),
  m_noFrames(noFrames),
  m_length(length),
  m_timeStamp(timeStamp),
  m_retryNo(retryNo)
{

}

UanHeaderRcRts::~UanHeaderRcRts()
{

}

TypeId
UanHeaderRcRts::GetTypeId()
{
  static TypeId tid = TypeId("ns3::UanHeaderRcRts")
    .SetParent<Header> ()
    .AddConstructor<UanHeaderRcRts>()
    ;
  return tid;

}

void
UanHeaderRcRts::SetFrameNo(uint8_t no)
{
  m_frameNo = no;
}

void
UanHeaderRcRts::SetNoFrames(uint8_t no)
{
  m_noFrames = no;
}

void
UanHeaderRcRts::SetLength(uint16_t length)
{
  m_length = length;
}
void
UanHeaderRcRts::SetTimeStamp(Time timeStamp)
{
  m_timeStamp = timeStamp;
}

void
UanHeaderRcRts::SetRetryNo(uint8_t no)
{
  m_retryNo = no;
}
uint8_t
UanHeaderRcRts::GetNoFrames()
{
  return m_noFrames;
}

uint16_t
UanHeaderRcRts::GetLength()
{
  return m_length;
}

Time
UanHeaderRcRts::GetTimeStamp(void)
{
  return m_timeStamp;
}

uint8_t
UanHeaderRcRts::GetRetryNo(void)
{
  return m_retryNo;
}

uint8_t
UanHeaderRcRts::GetFrameNo(void)
{
  return m_frameNo;
}

uint32_t
UanHeaderRcRts::GetSerializedSize (void) const
{
  return 1 + 1 + 1 + 4 + 2;
}

void
UanHeaderRcRts::Serialize (Buffer::Iterator start) const
{
  start.WriteU8(m_frameNo);
  start.WriteU8(m_retryNo);
  start.WriteU8(m_noFrames);
  start.WriteU16(m_length);
  start.WriteU32((uint32_t) (m_timeStamp.GetSeconds()*1000.0 + 0.5));
  //start.WriteU16(uint16_t (m_timeStamp.GetSeconds()*1000));
}

uint32_t
UanHeaderRcRts::Deserialize (Buffer::Iterator start)
{
  Buffer::Iterator rbuf = start;
  m_frameNo = rbuf.ReadU8();
  m_retryNo = rbuf.ReadU8();
  m_noFrames = rbuf.ReadU8();
  m_length = rbuf.ReadU16();
  m_timeStamp = Seconds ( ((double) rbuf.ReadU32())/1000.0 );
  //m_timeStamp = Seconds ( rbuf.ReadU16()/1000 );
  return rbuf.GetDistanceFrom(start);
}

void
UanHeaderRcRts::Print (std::ostream &os) const
{
  os << "Frame #=" << (uint32_t) m_frameNo << " Retry #=" << (uint32_t) m_retryNo << " Num Frames=" << (uint32_t) m_noFrames << "Length=" << m_length << " Time Stamp=" << m_timeStamp.GetSeconds();
}

TypeId
UanHeaderRcRts::GetInstanceTypeId(void) const
{
  return GetTypeId();
}


UanHeaderRcCts::UanHeaderRcCts() :
  Header(),
  m_frameNo(0),
  m_timeStamp1(Seconds(0)),
  m_timeStamp2(Seconds(0)),
  m_retryNo(0),
  m_delay(Seconds(0))
{

}

UanHeaderRcCts::UanHeaderRcCts(uint8_t frameNo, uint8_t retryNo, Time ts1, Time ts2, Time delay,
                              uint16_t rate, uint16_t rr, Time blockTime, Time windowTime):
  Header(),
  m_frameNo(frameNo),
  m_timeStamp1(ts1),
  m_timeStamp2(ts2),
  m_retryNo(retryNo),
  m_delay(delay),
  m_blockTime(blockTime),
  m_winTime(windowTime),
  m_retryRate(rr),
  m_rateNum(rate)
{

}

UanHeaderRcCts::~UanHeaderRcCts()
{

}

TypeId
UanHeaderRcCts::GetTypeId()
{
  static TypeId tid = TypeId("ns3::UanHeaderRcCts")
    .SetParent<Header> ()
    .AddConstructor<UanHeaderRcCts>()
    ;
  return tid;

}

void
UanHeaderRcCts::SetFrameNo(uint8_t frameNo)
{
  m_frameNo = frameNo;
}

void
UanHeaderRcCts::SetTimeStamp1(Time timeStamp)
{
  m_timeStamp1 = timeStamp;
}

void
UanHeaderRcCts::SetTimeStamp2(Time timeStamp)
{
  m_timeStamp2 = timeStamp;
}

void
UanHeaderRcCts::SetDelayToTx(Time delay)
{
  m_delay = delay;
}

void
UanHeaderRcCts::SetRetryNo(uint8_t no)
{
  m_retryNo = no;
}

void
UanHeaderRcCts::SetRateNum(uint16_t rate)
{
  m_rateNum = rate;
}

void
UanHeaderRcCts::SetRetryRate(uint16_t rate)
{
  m_retryRate = rate;
}

void
UanHeaderRcCts::SetBlockTime(Time t)
{
  m_blockTime = t;
}

void
UanHeaderRcCts::SetWindowTime(Time t)
{
  m_winTime = t;
}

uint16_t
UanHeaderRcCts::GetRetryRate(void)
{
  return m_retryRate;

}
uint8_t
UanHeaderRcCts::GetFrameNo()
{
  return m_frameNo;
}

Time
UanHeaderRcCts::GetTimeStamp1(void)
{
  return m_timeStamp1;
}

Time
UanHeaderRcCts::GetTimeStamp2(void)
{
  return m_timeStamp2;
}
Time
UanHeaderRcCts::GetDelayToTx(void)
{
  return m_delay;
}

uint8_t
UanHeaderRcCts::GetRetryNo()
{
  return m_retryNo;
}

uint16_t
UanHeaderRcCts::GetRateNum()
{
  return m_rateNum;
}
uint32_t
UanHeaderRcCts::GetSerializedSize (void) const
{
  return 1 + 1 + 4 + 4 + 4 + 2 + 2 + 2 + 2;
}

Time
UanHeaderRcCts::GetBlockTime(void)
{
  return m_blockTime;
}
Time
UanHeaderRcCts::GetWindowTime(void)
{
  return m_winTime;
}

void
UanHeaderRcCts::Serialize (Buffer::Iterator start) const
{
  start.WriteU8(m_frameNo);
  start.WriteU8(m_retryNo);
  start.WriteU32((uint32_t) (m_timeStamp1.GetSeconds()*1000.0 + 0.5));
  start.WriteU32((uint32_t) (m_timeStamp2.GetSeconds()*1000.0 + 0.5) );
  start.WriteU32((uint32_t) (m_delay.GetSeconds()*1000.0 + 0.5));
  start.WriteU16(m_rateNum);
  start.WriteU16(m_retryRate);
  start.WriteU16((uint16_t) (m_blockTime.GetSeconds()*1000.0 + 0.5));
  start.WriteU16((uint16_t) (m_winTime.GetSeconds()*1000.0 + 0.5));
}

uint32_t
UanHeaderRcCts::Deserialize (Buffer::Iterator start)
{
  Buffer::Iterator rbuf = start;
  m_frameNo = rbuf.ReadU8();
  m_retryNo = rbuf.ReadU8();
  m_timeStamp1 = Seconds ( ( (double) rbuf.ReadU32()) / 1000.0 );
  m_timeStamp2 = Seconds ( ( (double) rbuf.ReadU32()) / 1000.0 );
  m_delay = Seconds ( ( (double) rbuf.ReadU32()) / 1000.0 );
  m_rateNum = rbuf.ReadU16();
  m_retryRate = rbuf.ReadU16();
  m_blockTime = Seconds ( ( (double) rbuf.ReadU16()) / 1000.0 );
  m_winTime = Seconds ( ( (double) rbuf.ReadU16()) / 1000.0 );
  return rbuf.GetDistanceFrom(start);
}

void
UanHeaderRcCts::Print (std::ostream &os) const
{
  os << "CTS (Frame #=" << (uint32_t) m_frameNo << " Retry #=" << (uint32_t) m_retryNo << " Time Stamp 1=" << m_timeStamp1.GetSeconds() << "Time Stamp 2=" << m_timeStamp2.GetSeconds() << " Delay until TX=" << m_delay.GetSeconds() << " Retry Rate=" << m_retryRate << " Block time=" << m_blockTime.GetSeconds() << " Window time=" << m_winTime.GetSeconds() << ")";
}

TypeId
UanHeaderRcCts::GetInstanceTypeId(void) const
{
  return GetTypeId();
}

UanHeaderRcAck::UanHeaderRcAck()
  : m_frameNo(0)
{
}

UanHeaderRcAck::~UanHeaderRcAck()
{
}

TypeId
UanHeaderRcAck::GetTypeId()
{
  static TypeId tid = TypeId("ns3::UanHeaderRcAck")
    .SetParent<Header> ()
    .AddConstructor<UanHeaderRcAck>()
    ;
  return tid;
}

void
UanHeaderRcAck::SetFrameNo(uint8_t noFrames)
{
  m_frameNo = noFrames;
}

void
UanHeaderRcAck::AddNackedFrame(uint8_t frame)
{
  m_nackedFrames.insert(frame);
}

const std::set<uint8_t> &
UanHeaderRcAck::GetNackedFrames(void) const
{
  return m_nackedFrames;
}

uint8_t
UanHeaderRcAck::GetFrameNo(void) const
{
  return m_frameNo;
}

uint8_t
UanHeaderRcAck::GetNoNacks(void) const
{
  return m_nackedFrames.size();
}

uint32_t
UanHeaderRcAck::GetSerializedSize (void) const
{
  return 1 + 1 + GetNoNacks();
}

void
UanHeaderRcAck::Serialize (Buffer::Iterator start) const
{
  start.WriteU8(m_frameNo);
  start.WriteU8(GetNoNacks());
  std::set<uint8_t>::iterator it = m_nackedFrames.begin();
  for(;it != m_nackedFrames.end();it++)
    {
      start.WriteU8(*it);
    }
}

uint32_t
UanHeaderRcAck::Deserialize (Buffer::Iterator start)
{
  Buffer::Iterator rbuf = start;
  m_frameNo = rbuf.ReadU8();
  uint8_t noAcks = rbuf.ReadU8();
  m_nackedFrames.clear();
  for(uint32_t i=0;i<noAcks;i++)
    {
      m_nackedFrames.insert(rbuf.ReadU8());
    }
  return rbuf.GetDistanceFrom(start);
}

void
UanHeaderRcAck::Print (std::ostream &os) const
{
  os << "# Frames=" << (uint32_t) m_frameNo << " # nacked=" << (uint32_t) GetNoNacks() << " Nacked: ";
  if(GetNoNacks() > 0)
    {
      std::set<uint8_t>::iterator it = m_nackedFrames.begin();
      os << (uint32_t) *it;
      it++;
      for(; it != m_nackedFrames.end();it++)
        os << ", " << (uint32_t) *it;
    }
}

TypeId
UanHeaderRcAck::GetInstanceTypeId(void) const
{
  return GetTypeId();
}

} //namespace ns3
