#include "uan-mac-aloha.h"
#include "uan-tx-mode.h"
#include "uan-address.h"
#include "ns3/log.h"
#include "uan-phy.h"
#include "uan-header-common.h"

#include <iostream>
NS_LOG_COMPONENT_DEFINE("UanMacAloha");


namespace ns3
{

NS_OBJECT_ENSURE_REGISTERED (UanMacAloha);

UanMacAloha::UanMacAloha()
{

  //m_address = UanAddress::Allocate();
  NS_LOG_DEBUG("Assigned address " << m_address);
}

UanMacAloha::~UanMacAloha()
{
}

TypeId
UanMacAloha::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::UanMacAloha")
    .SetParent<Object> ()
    .AddConstructor<UanMacAloha>()
    ;
  return tid;

//    .AddAttribute ("Address",
//                   "This node's MAC Address",



}
Address
UanMacAloha::GetAddress(void)
{
  return m_address;
}

void
UanMacAloha::SetAddress(UanAddress addr)
{
  m_address=addr;
}
bool
UanMacAloha::Enqueue(Ptr<Packet> packet, const Address &dest, uint16_t protocolNumber)
{
  NS_LOG_DEBUG("" << Simulator::Now().GetSeconds() << " MAC " << UanAddress::ConvertFrom(GetAddress()) << " Queueing packet for " << UanAddress::ConvertFrom(dest));

  if(!m_phy->IsStateTx())
    {
      UanAddress src = UanAddress::ConvertFrom(GetAddress());
      UanAddress udest = UanAddress::ConvertFrom(dest);

      //UanHeaderCommon header(src, dest, (uint8_t) 0, (uint16_t) packet->GetSize(), Simulator::Now());
      UanHeaderCommon header;
      header.SetSrc(src);
      header.SetDest(udest);
      header.SetType(0);

      packet->AddHeader(header);
      //std::cout << "Packet enqueue at address " << GetAddress() << ": " << *packet << std::endl;
      m_phy->SendPacket(packet, 0);
      return true;
    }
  else
    return false;
}

void
UanMacAloha::SetForwardUpCb(Callback<void, Ptr<Packet>, const UanAddress& > cb)
{
  this->m_forUpCb = cb;
}
void
UanMacAloha::AttachPhy(Ptr<UanPhy> phy)
{
  m_phy = phy;
  phy->SetReceiveOkCallback(MakeCallback(&UanMacAloha::RxPacketGood, this));
  phy->SetReceiveErrorCallback(MakeCallback(&UanMacAloha::RxPacketError, this));

}
void
UanMacAloha::RxPacketGood(Ptr<Packet> pkt, double sinr, UanTxMode txMode)
{
  //TODO:  Find the source!
  //TODO:  Adding a MAC header would do it!

  UanHeaderCommon header;
  pkt->RemoveHeader(header);
  NS_LOG_DEBUG("Receiving packet from " << header.GetSrc() << " For " << header.GetDest());

  if(header.GetDest() == GetAddress())
    {
      NS_LOG_DEBUG("That means this is for me :)");
      m_forUpCb(pkt, header.GetDest());
    }

}

void
UanMacAloha::RxPacketError(Ptr<Packet> pkt, double sinr)
{
  NS_LOG_DEBUG("" << Simulator::Now() << " MAC " << UanAddress::ConvertFrom(GetAddress()) << " Received packet in error with sinr " << sinr);
}

Address
UanMacAloha::GetBroadcast(void) const
{
  UanAddress broadcast(255);
  return broadcast;
}

}
