/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 *
 *
 */

#ifndef UANMACALOHA_H_
#define UANMACALOHA_H_

#include "uan-mac.h"
#include "uan-address.h"

namespace ns3
{


class UanPhy;
class UanTxMode;

class UanMacAloha : public ns3::UanMac
{
public:
  UanMacAloha();
  virtual ~UanMacAloha();
  static TypeId GetTypeId(void);


  //Inheritted functions
  Address GetAddress(void);
  virtual void SetAddress(UanAddress addr);
  virtual bool Enqueue(Ptr<Packet> packet, const Address &dest, uint16_t protocolNumber);
  virtual void SetForwardUpCb(Callback<void, Ptr<Packet>, const UanAddress& > cb);
  virtual void AttachPhy(Ptr<UanPhy> phy);
  virtual Address GetBroadcast(void) const;

private:
  UanAddress m_address;
  Ptr<UanPhy> m_phy;
  Callback<void, Ptr<Packet>, const UanAddress& > m_forUpCb;

  /**
   * \brief Receive packet from lower layer (passed to PHY as callback)
   * \param pkt Packet being received
   * \param sinr SINR of received packet
   * \param txMode Mode of received packet
   */
  void RxPacketGood(Ptr<Packet> pkt, double sinr, UanTxMode txMode);

  /**
   * \brief Packet received at lower layer in error
   * \param pkt Packet received in error
   * \param sinr SINR of received packet
   */
  void RxPacketError(Ptr<Packet> pkt, double sinr);
};

}

#endif /*UANMACALOHA_H_*/
