/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 *
 *
 */
#ifndef UANMACCW_H_
#define UANMACCW_H_

#include "ns3/uan-mac.h"
#include "ns3/nstime.h"
#include "ns3/simulator.h"
#include "ns3/uan-phy.h"
#include "ns3/uan-tx-mode.h"
#include "ns3/uan-address.h"


namespace ns3
{

class UanMacCw: public ns3::UanMac, public ns3::UanPhyListener
{
public:
  UanMacCw();
  virtual ~UanMacCw();
  static TypeId GetTypeId(void);

  /**
   * \param cw Contention window size
   */
  virtual void SetCw(uint32_t cw);
  /**
   * \param duration Slot time duration
   */
  virtual void SetSlotTime(Time duration);
  /**
   * \returns Contention window size
   */
  virtual uint32_t GetCw(void);
  /**
   * \returns slot time duration
   */
  virtual Time GetSlotTime(void);

  //Inherited methods
  virtual Address GetAddress();
  virtual void SetAddress(UanAddress addr);
  virtual bool Enqueue(Ptr<Packet> packet, const Address &dest, uint16_t protocolNumber);
  virtual void SetForwardUpCb(Callback<void, Ptr<Packet>, const UanAddress&> cb);
  virtual void AttachPhy(Ptr<UanPhy> phy);
  virtual Address GetBroadcast(void) const;

  //PHY listeners
  virtual void NotifyRxStart(void);
  virtual void NotifyRxEndOk(void);
  virtual void NotifyRxEndError(void);
  virtual void NotifyCcaStart(void);
  virtual void NotifyCcaEnd(void);
  virtual void NotifyTxStart(Time duration);
private:
  typedef enum {IDLE, CCABUSY, RUNNING, TX} State;

  Callback <void, Ptr<Packet>, const UanAddress& > m_forwardUpCb;
  UanAddress m_address;
  Ptr<UanPhy> m_phy;
  TracedCallback<Ptr<const Packet>, UanTxMode > m_rxLogger;
  TracedCallback<Ptr<const Packet>, uint16_t  > m_enqueueLogger;
  TracedCallback<Ptr<const Packet>, uint16_t  > m_dequeueLogger;

  //Mac parameters
  uint32_t m_cw;
  Time m_slotTime;

  //State variables
  Time m_sendTime;
  Time m_savedDelayS;
  Ptr<Packet> m_pktTx;
  uint16_t m_pktTxProt;
  EventId m_sendEvent;
  EventId m_txEndEvent;
  State m_state;

  void PhyRxPacketGood(Ptr<Packet> packet, double sinr, UanTxMode mode);
  void PhyRxPacketError(Ptr<Packet> packet, double sinr);
  void SaveTimer(void);
  void StartTimer(void);
  void SendPacket(void);
  void EndTx(void);
};

}

#endif /* UANMACCW_H_ */
