/*
 * uan-noise-model-default.cc
 *
 *  Created on: 10-Oct-2008
 *      Author: ltracy
 */

#include "uan-noise-model-default.h"
#include "ns3/double.h"

#include <cmath>
namespace ns3
{
NS_OBJECT_ENSURE_REGISTERED(UanNoiseModelDefault);

UanNoiseModelDefault::UanNoiseModelDefault()
{

}

UanNoiseModelDefault::~UanNoiseModelDefault()
{
}

TypeId
UanNoiseModelDefault::GetTypeId(void)
{
  static TypeId tid = TypeId("ns3::UanNoiseModelDefault")
    .SetParent<Object>()
    .AddConstructor<UanNoiseModelDefault>()
    .AddAttribute("Wind", "Wind speed in m/s",
        DoubleValue(5),
        MakeDoubleAccessor(&UanNoiseModelDefault::m_wind),
        MakeDoubleChecker<double>(0))
    .AddAttribute("Shipping", "Shipping contribution to noise between 0 and 1",
        DoubleValue(0),
        MakeDoubleAccessor(&UanNoiseModelDefault::m_shipping),
        MakeDoubleChecker<double>(0,1))
    ;

  return tid;
}

double
UanNoiseModelDefault::GetNoiseDbHz(double fKhz) const
{
  double turb, win, ship, thermal, noise;
  turb = 17.0 - 30.0 * log10(fKhz);
  turb = pow(10.0, turb*0.1);

  ship = 40.0 + 20.0 *(this->m_shipping-0.5) + 26.0*log10(fKhz) - 60.0*log10(fKhz+0.03);
  ship = pow(10.0,(ship*0.01));

  win = 50.0+7.5*pow(m_wind, 0.5) + 20.0*log10(fKhz) - 40.0*log10(fKhz+0.4);
  win = pow(10.0, m_wind*0.1);

  thermal = -15 + 20*log10(fKhz);
  thermal = pow(10, thermal*0.1);

  noise = 10*log10(turb + ship + win + thermal);

  return noise;
}

} //namespace ns3
