/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 *
 *
 */

#ifndef UANPHYSIMPLE_H_
#define UANPHYSIMPLE_H_


#include "ns3/uan-phy.h"
#include "ns3/uan-tx-mode.h"
#include "ns3/traced-callback.h"
#include <list>

namespace ns3
{

/**
 * \class UanPhySimple
 * \brief Simple Phy model.  This model is outdated.  UanPhyGen should be used.
 */
class UanPhySimple : public ns3::UanPhy
{
public:

  UanPhySimple();
  virtual ~UanPhySimple();

  static TypeId GetTypeId(void);
  virtual void SendPacket(Ptr<Packet> pkt, uint32_t modeNum);
  virtual void RegisterListener(UanPhyListener *listener);
  virtual void StartRxPacket(Ptr<Packet> pkt, double rxPowerDb, UanTxMode txMode, UanPdp pdp);
  virtual void SetReceiveOkCallback (RxOkCallback cb);
  virtual void SetReceiveErrorCallback (RxErrCallback cb);
  virtual bool IsStateIdle(void);
  virtual bool IsStateBusy(void);
  virtual bool IsStateRx(void);
  virtual bool IsStateTx(void);
  virtual bool IsStateCcaBusy(void);
  virtual void SetRxGainDb(double gain);
  virtual void SetTxPowerDb(double txpwr);
  virtual void SetRxThresholdDb(double thresh);
  virtual void SetCcaThresholdDb(double thresh);
  virtual double GetRxGainDb(void);
  virtual double GetTxPowerDb(void);
  virtual double GetRxThresholdDb(void);
  virtual double GetCcaThresholdDb(void);
  virtual Ptr<UanChannel> GetChannel(void) const;
  virtual Ptr<UanNetDevice> GetDevice(void);
  virtual Ptr<UanTransducer> GetTransducer(void);
  virtual void SetChannel(Ptr<UanChannel> channel);
  virtual void SetDevice(Ptr<UanNetDevice> device);
  virtual void SetMac(Ptr<UanMac> mac);
  virtual void SetTransducer(Ptr<UanTransducer> trans);
  virtual void NotifyTransStartTx(Ptr<Packet> packet, double txPowerDb, UanTxMode txMode);
  virtual void NotifyIntChange(void);
  virtual uint32_t GetNModes(void);
  virtual UanTxMode GetMode(uint32_t n);
  virtual Ptr<Packet> GetPacketRx(void) const;
private:
  typedef std::list<UanPhyListener *> ListenerList;

  UanModesList m_modes;

  State m_state;
  ListenerList m_listeners;
  RxOkCallback m_recOkCb;
  RxErrCallback m_recErrCb;
  Ptr<UanChannel> m_channel;
  Ptr<UanTransducer> m_transducer;
  Ptr<UanNetDevice> m_device;
  Ptr<UanMac> m_mac;
  double m_rxGainDb;
  double m_txPwrDb;
  double m_rxThreshDb;
  double m_ccaThreshDb;

  Ptr<Packet> m_pktRx;
  double m_minRxSinrDb;
  double m_rxRecvPwrDb;

  TracedCallback<Ptr<const Packet>, double, UanTxMode> m_rxOkLogger;
  TracedCallback<Ptr<const Packet>, double, UanTxMode> m_rxErrLogger;
  TracedCallback<Ptr<const Packet>, double, UanTxMode> m_txLogger;


  double CalculateSinrDb(Ptr<Packet> pkt, double rxPowerDb);
  double GetInterferenceDb(Ptr<Packet> pkt);
  double DbToKp(double db);
  double KpToDb(double kp);
  void RxEndEvent(Ptr<Packet> pkt, double rxPowerDb, UanTxMode txMode);
  void TxEndEvent();

  void NotifyListenersRxStart(void);
  void NotifyListenersRxGood(void);
  void NotifyListenersRxBad(void);
  void NotifyListenersCcaStart(void);
  void NotifyListenersCcaEnd(void);
  void NotifyListenersTxStart(Time duration);


};

} //namespace ns3

#endif /*UANPHYSIMPLE_H_*/
