/*
 * uan-prop-model-thorp.cc
 *
 *  Created on: 3-Nov-2008
 *      Author: ltracy
 */

#include "uan-prop-model-thorp.h"
#include "uan-tx-mode.h"
#include "ns3/double.h"
#include "ns3/log.h"

NS_LOG_COMPONENT_DEFINE("UanPropModelThorp");

namespace ns3
{

NS_OBJECT_ENSURE_REGISTERED(UanPropModelThorp);

UanPropModelThorp::UanPropModelThorp()
{
  // TODO Auto-generated constructor stub

}

UanPropModelThorp::~UanPropModelThorp()
{
  // TODO Auto-generated destructor stub
}

TypeId
UanPropModelThorp::GetTypeId(void)
{
  static TypeId tid = TypeId("ns3::UanPropModelThorp")
    .SetParent<Object>()
    .AddConstructor<UanPropModelThorp>()
    .AddAttribute("SpreadCoef",
                  "Spreading coefficient used in calculation of Thorp's approximation",
                  DoubleValue(1.5),
                  MakeDoubleAccessor(&UanPropModelThorp::m_SpreadCoef),
                  MakeDoubleChecker<double>())
    ;
  return tid;
}

double
UanPropModelThorp::GetPathLossDb(Ptr<MobilityModel> a, Ptr<MobilityModel> b, UanTxMode mode)
{
  double distKyd = a->GetDistanceFrom(b) / 1.093613298; //Convert from dB/km to dB/kyd

  return m_SpreadCoef * 10.0 * log10(distKyd)
         + distKyd * GetAttenDbKyd(mode.GetCenterFreqHz()/1000.0);
}

UanPdp
UanPropModelThorp::GetPdp(Ptr<MobilityModel> a, Ptr<MobilityModel> b, UanTxMode mode)
{
  return UanPdp::CreateImpulsePdp();
}

Time
UanPropModelThorp::GetDelay(Ptr<MobilityModel> a, Ptr<MobilityModel> b, UanTxMode mode)
{
  return Seconds(a->GetDistanceFrom(b) / 1500.0);
}

double
UanPropModelThorp::GetAttenDbKyd(double freqKhz)
{

  double fsq = freqKhz*freqKhz;
  double atten;

  if(freqKhz < 0.4)
    {
      atten = 0.11 * fsq/(1+fsq) + 44 * fsq/(4100 + freqKhz)
              + 2.75 * 0.0001 * fsq + 0.003;
    }
  else
    {
      atten = 0.002 + 0.11 * (freqKhz / (1 + freqKhz)) + 0.011*freqKhz;
    }

  return atten;
}

} //namespace ns3
