/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 */

#include <iostream>
#include <fstream>

#include "ns3/core-module.h"
#include "ns3/common-module.h"
#include "ns3/uan-module.h"
#include "ns3/node-module.h"
#include "ns3/mobility-module.h"
#include "ns3/contrib-module.h"
#include "ns3/helper-module.h"

using namespace ns3;
using namespace std;
void SendPacket(Ptr<NetDevice> dev, int, int);
void GenerateTraffic(NetDeviceContainer devlist, double offeredLoad, int pktSize, double dataRate, double simSeconds);

NS_LOG_COMPONENT_DEFINE("Main");



int
main()
{
  //LogComponentEnable("UanHelper", LOG_LEVEL_ALL);
  //LogComponentEnable("UanChannel", LOG_LEVEL_ALL);
  //LogComponentEnable("UanPhyGen", LOG_LEVEL_ALL);
  //LogComponentEnable("UanMacCw", LOG_LEVEL_ALL);
  LogComponentEnable("UanPropModelBh", LOG_LEVEL_ALL);
  UanHelper uan;
  uan.SetMac("ns3::UanMacAloha");
  uan.SetPhy("ns3::UanPhyGen");

  Ptr<UanPropModelBh> prop = CreateObject<UanPropModelBh>("ConfigFile", StringValue("exbhconfig.cfg"));
  Ptr<UanChannel>  channel = CreateObject<UanChannel>("PropagationModel", PointerValue(prop));

  NodeContainer nodes;
  nodes.Create(10);

  NetDeviceContainer devices = uan.Install(nodes, channel);


  MobilityHelper mobility;
  mobility.SetPositionAllocator("ns3::GridPositionAllocator",
			       "MinX", DoubleValue(0.0),
			       "MinY", DoubleValue(0.0),
			       "DeltaX", DoubleValue(50.0),
			       "DeltaY", DoubleValue(50.0),
			       "GridWidth", UintegerValue(3),
			       "LayoutType", StringValue ("RowFirst"));
  mobility.SetMobilityModel("ns3::StaticMobilityModel");

  mobility.Install(nodes);

  ofstream ascii("secondtrace.tr");
  uan.EnableAsciiAll(ascii);
  //  uan.EnablePcapAll("secondtrace.pcap");

  GenerateTraffic(devices, 1.0, 32, 80, 2000);
  Simulator::Run();
}



void
GenerateTraffic(NetDeviceContainer devlist, double offeredLoad, int pktSize, double dataRate, double simSeconds)
{
  int nodeCt = devlist.GetN();
  double txDelay = ((double) pktSize) * 8.0 /dataRate;
  double meanArr = txDelay/offeredLoad;
  ExponentialVariable exp(meanArr);
  UniformVariable nodeVar(0, nodeCt);

  double time=0.0;

  while(time<simSeconds)
    {

      double interTime = exp.GetValue();
      int src = (int) nodeVar.GetValue();
      int dest=0;
      do
	{
	  dest = (int) nodeVar.GetValue();
	}
      while(dest==src);
      time += interTime;

      Simulator::Schedule (Seconds(time), &SendPacket, devlist.Get(src), pktSize, dest);
    }

}

void
SendPacket(Ptr<NetDevice> dev, int size, int dest)
{
  dev->Send(Create<Packet>(size), UanAddress(dest), 0);
}
