/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Leonard Tracy <lentracy@u.washington.edu>
 *
 *
 */


//TODO:  Implement pertubations and averaging.
//NOTE:  ^^^ THIS HAS NOT YET BEEN DONE

#include "ns3/common-module.h"
#include "ns3/core-module.h"
#include "ns3/uan-module.h"
//#include "ns3/assert.h"

//#include "ns3/log.h"

#define USE_SQL_DAT

#include <iostream>
#include <fstream>
#include <algorithm>
#include <cctype>
#include <string>
#include <sstream>
#include <set>
#include <vector>
#include <iomanip>
#ifdef USE_SQL_DAT
#include <sqlite3.h>
#endif

using namespace ns3;
using namespace std;




NS_LOG_COMPONENT_DEFINE("BhCreateDat");

static int
sqlCallback(void *vd, int32_t argc, char **argv, char **cname)
{
  for(int32_t i=0;i<argc;i++)
    {
      std::cout << "Column: " << cname[i] << " is " << argv[i] << std::endl;
    }
  return 0;
}
class DatMaker : public BhConfig
{

public:
  void PrintHelp(void);
  void MakeDatFiles(void);
  virtual bool Config(const char *cfgFile);

private:
  typedef std::list<pair<double, double> > SspList;
  //pair < delay in seconds, amplitude >
  typedef std::vector<pair<double, complex<double> > > TapVec;
  string m_BhPath;
  uint32_t m_Nmedia;
  string m_Ssopt;
  uint32_t m_Nmesh;
  uint32_t m_TopSigma;
  double m_Zdepth;

  SspList m_sspVals;
  string m_BotSigma;
  double m_BotRough;

  string m_BotArgs;
  string m_RunType;

  uint32_t m_NumBeams;
  double m_MinBeamAngle;
  double m_MaxBeamAngle;

  double m_Step;
  double m_Zbox;
  double m_Rbox;

  double m_PerturbMean;
  double m_PerturbVariance;
  uint32_t m_NumAverages;

  uint32_t m_PreTaps;
  uint32_t m_TotalTaps;
  double m_resolution;

  void WriteEnvFile(string filename, uint32_t freqHz);
  void WriteEnvFile(string filename, uint32_t freqHz, SspList ssp);
  void ReadArrivals(list<BellhopResp> &results);
  set<string> FillParams(void);

#ifdef USE_SQL_DAT
  void WriteResults(list<BellhopResp> &results, sqlite3 *sqldb);
  sqlite3 *CreateDb(string dbfile);
#else
  void WriteResults(list<BellhopResp> &results);
#endif
  list<TapVec> SumTapVecList(list<TapVec> &a, list<TapVec> &b);
  list<DatMaker::TapVec> ProcessTaps(list<BellhopResp> &results);
  SspList PerturbSsp(void);

};

void
DatMaker::PrintHelp(void)
{
  cout << "Usage: create-dat <CFG_FILE>" << endl;
  cout << "\tCFG_FILE=Configuration file describing frequency, depth, and ranges to include" << endl;
  cout << "See documentation for details" << endl << endl;
}

void
DatMaker::WriteEnvFile(string filename, uint32_t freqHz)
{
  WriteEnvFile(filename, freqHz, m_sspVals);
}
void
DatMaker::WriteEnvFile(string filename, uint32_t freqHz, SspList ssp)
{

  ofstream env(filename.c_str());

  env << "'CreateDatEnv'" << endl;
  env << freqHz << endl << m_Nmedia << endl << m_Ssopt << endl;
  env << m_Nmesh << " " << m_TopSigma << " " << m_Zdepth << endl;

  list<pair<double, double> >::iterator it = ssp.begin();

  for(;it != ssp.end(); it++)
    {
      env << (*it).first << " " << (*it).second << " /" << endl;
    }

  env << m_BotSigma << " " << m_BotRough << endl;
  env << m_BotArgs << endl;

  //Source depths"
  env << m_NumSrcDepth << endl;
  for(uint32_t i=0;i<m_NumSrcDepth;i++)
    {
      env << m_MinSrcDepthM + m_SrcDepthStepM*i << " ";
    }
  env << endl;
  //Rx depths
  env << m_NumRecDepth << endl;
  for(uint32_t i=0;i<m_NumRecDepth;i++)
    {
      env << m_MinRecDepthM + m_RecDepthStepM*i << " ";
    }
  env << endl;

  //Ranges
  env << m_NumRanges << endl;
  for(uint32_t i=0;i<m_NumRanges;i++)
    {
      env << m_MinRangeKm + i*m_RangeStepKm << " ";
    }
  env << endl;

  env << m_RunType << endl;
  env << m_NumBeams << endl << m_MinBeamAngle << " " << m_MaxBeamAngle << " /"  << endl;
  env << m_Step << " " << m_Zbox << " " << m_Rbox << endl;
  env << endl;
  env.close();
}

set<string>
DatMaker::FillParams(void)
{

  set<string> params;

  params.insert("bellhoppath");
  params.insert("SSP");
  params.insert("nmedia");
  params.insert("ssopt");
  params.insert("nmesh");
  params.insert("topsigma");
  params.insert("sspbottomdepth");
  params.insert("bottomsigma");
  params.insert("bottomroughness");
  params.insert("runtype");
  params.insert("numbeams");
  params.insert("minbeamangle");
  params.insert("maxbeamangle");
  params.insert("step");
  params.insert("raybottomdepth");
  params.insert("raymaxrange");
  params.insert("perturbmean");
  params.insert("perturbvariance");
  params.insert("numavgs");
  params.insert("tapsbeforemax");
  params.insert("totaltaps");
  params.insert("bottomargument");
  params.insert("resolution");

  return params;
}


bool
DatMaker::Config(const char *cfgFileName)
{

  ifstream inf(cfgFileName);
  ostringstream buf;
  int (*tl)(int) = tolower;

  if(!BhConfig::Config(cfgFileName))
    {
      NS_FATAL_ERROR("Bad configuration file:  Missing basic params");
      return false;
    }


  set<string> params = FillParams();


  if(!inf.good())
    {
      return false; //Bad environment file
    }
  m_ConfigFile = string(cfgFileName);

  string line;
  uint32_t lineNo=0;

  while(getline(inf, line))
    {
      lineNo++;
      trim(line);
      //NS_LOG_DEBUG("Line " << lineNo << ": " << line);
      if(line == "SSP:")
        {
          params.erase("SSP");
          buf.str("");
          while(1)
            {

              if(!getline(inf, line))
                {
                  NS_FATAL_ERROR("Found no ENDSSP: after SSP: in config file " << m_ConfigFile);
                }
              if(line == "ENDSSP:")
                {
                  break;
                }
              buf << line << endl;
              uint32_t comPos = line.find_first_of(",");
              if(comPos == string::npos)
                {
                  NS_FATAL_ERROR("Invalid SSP entry: " << line);
                }
              string firstArg = trim(line.substr(0,comPos));
              string scdArg = trim(line.substr(comPos+1, line.length() - comPos + 1));
              m_sspVals.push_back(make_pair(atof(firstArg.c_str()), atof(scdArg.c_str())));
            }
        }
      else
        {
          if(line.empty() || line.at(0) == '#') //comment
            continue;

          uint32_t colonPos = line.find_first_of(':');
          if(colonPos == string::npos)
            {
              NS_LOG_WARN("WARNING: Unrecognized configuration line at line number " << lineNo << ": " << line);
              continue;
            }


          string configParam = trim(line.substr(0, colonPos));
          transform(configParam.begin(), configParam.end(), configParam.begin(), tl);


          uint32_t endPos = line.find_first_of("# ", colonPos+1);
          uint32_t valSize = (endPos == string::npos)?(line.length() - colonPos + 1):(colonPos - endPos - 1);
          string configValue = trim(line.substr(colonPos+1, valSize));

          params.erase(configParam);
          if(configParam == "bellhoppath")
            {
              m_BhPath = configValue;
              NS_LOG_DEBUG("Set BH path: " << m_BhPath);
            }
          else if(configParam == "nmedia")
            {
              m_Nmedia = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set NMedia: " << m_Nmedia);
            }
          else if(configParam == "ssopt")
            {
              m_Ssopt = configValue;
              NS_LOG_DEBUG("Set SSOPT: " << m_Ssopt);
            }
          else if(configParam == "nmesh")
            {
              m_Nmesh = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set NMesh:" << m_Nmesh);
            }
          else if(configParam == "topsigma")
            {
              m_TopSigma = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set top SIGMA: " << m_TopSigma);
            }
          else if(configParam == "sspbottomdepth")
            {
              m_Zdepth = atof(configValue.c_str());
              NS_LOG_DEBUG("Set SSP bottom depth: " << m_Zdepth << " m");
            }
          else if(configParam == "bottomsigma")
            {
              m_BotSigma = configValue;
              NS_LOG_DEBUG("Set bottom sigma: " << m_BotSigma);
            }
          else if(configParam == "bottomroughness")
            {
              m_BotRough = atof(configValue.c_str());
              NS_LOG_DEBUG("Set bottom roughness: " << m_BotRough);
            }
          else if(configParam == "bottomargument")
            {
              m_BotArgs = configValue;
              NS_LOG_DEBUG("Set bottom arguments");
            }
          else if(configParam == "runtype")
            {
              m_RunType = configValue;
              NS_LOG_DEBUG("Set run type: " << m_RunType);
            }
          else if(configParam == "numbeams")
            {
              m_NumBeams = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set number of beams: " << m_NumBeams);
            }
          else if(configParam == "minbeamangle")
            {
              m_MinBeamAngle = atof(configValue.c_str());
              NS_LOG_DEBUG("Set minimum beam angle: " << m_MinBeamAngle << " degrees");
            }
          else if(configParam == "maxbeamangle")
            {
              m_MaxBeamAngle = atof(configValue.c_str());
              NS_LOG_DEBUG("Set maximum beam angle: " << m_MaxBeamAngle << " degrees");
            }
          else if(configParam == "step")
            {
              m_Step = atof(configValue.c_str());
              NS_LOG_DEBUG("Set step: " << m_Step << " m");
            }
          else if(configParam == "raybottomdepth")
            {
              m_Zbox = atof(configValue.c_str());
              NS_LOG_DEBUG("Set ray bottom depth: " << m_Zbox << "m");
            }
          else if(configParam == "raymaxrange")
            {
              m_Rbox = atof(configValue.c_str());
              NS_LOG_DEBUG("Set max range to trace a ray: " << m_Rbox << " km");
            }
          else if(configParam == "perturbmean")
            {
              m_PerturbMean = atof(configValue.c_str());
              NS_LOG_DEBUG("Set mean for SSP petrubations: " << m_PerturbMean);
            }
          else if(configParam == "perturbvariance")
            {
              m_PerturbVariance = atof(configValue.c_str());
              NS_LOG_DEBUG("Set variance for SSP pertubations: " << m_PerturbVariance);
            }
          else if(configParam == "numavgs")
            {
              m_NumAverages = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set number of averages: " << m_NumAverages);
            }
          else if(configParam == "tapsbeforemax")
            {
              m_PreTaps = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set taps before max to keep: " << m_PreTaps << " taps");
            }
          else if(configParam == "totaltaps")
            {
              m_TotalTaps = atoi(configValue.c_str());
              NS_LOG_DEBUG("Set total taps: " << m_TotalTaps << " taps");
            }
          else if(configParam == "resolution")
            {
              m_resolution = atof(configValue.c_str());
            }
          else
            continue;

          params.erase(configParam);

        }
    }

  if(!params.empty())
    {
      NS_LOG_WARN("Config file " << m_ConfigFile << " is incomplete");
      NS_LOG_WARN("Missing parameters:");
      set<string>::iterator it = params.begin();
      for(; it != params.end(); it++)
        {
          NS_LOG_WARN(*it);
        }
      NS_FATAL_ERROR("Missing parameters");
      return false;
    }
  return true;
}
int main(int argc, char *argv[])
{
  LogComponentEnable("BhCreateDat", LOG_LEVEL_ALL);
  LogComponentEnable("UanPropBh", LOG_LEVEL_ALL);
  DatMaker bhc;

  if(argc != 2)
    {
      bhc.PrintHelp();
      return 1;
    }

  bhc.Config(argv[1]);
  //bhc.WriteEnvFile("TestEnv.env");
  //list<BellhopResp> results;
  //bhc.ReadArrivals(results);
  bhc.MakeDatFiles();

}

list<DatMaker::TapVec>
DatMaker::SumTapVecList(list<TapVec> &a, list<TapVec> &b)
{
  NS_ASSERT(a.size() == b.size());
  list<TapVec> out;

  list<TapVec>::iterator ait = a.begin();
  list<TapVec>::iterator bit = b.begin();
  for(;ait != a.end();ait++, bit++)
    {
      NS_ASSERT(ait->size() == bit->size());
      TapVec outvec(ait->size());
      for(uint32_t i=0;i < ait->size();i++)
        {
          TapVec &av = *ait;
          TapVec &bv = *bit;
          outvec[i] = make_pair(av[i].first, abs(av[i].second) + abs(bv[i].second));
        }
      out.push_back(outvec);
    }
  return out;
}

DatMaker::SspList
DatMaker::PerturbSsp(void)
{
  NormalVariable nrv(m_PerturbMean, m_PerturbVariance);

  SspList outlist;
  for(SspList::iterator it = m_sspVals.begin();it != m_sspVals.end();it++)
    {
      outlist.push_back(make_pair(it->first, it->second + nrv.GetValue()));
    }
  return outlist;
}

void
DatMaker::MakeDatFiles(void)
{
  string envFile("make-dat-envfile.env");
  ostringstream bhcommand;
  bhcommand << m_BhPath << " < " << envFile << " > /dev/null";

#ifdef USE_SQL_DAT
  sqlite3 *sqldb = CreateDb("sqldbtest.db");
#endif

  for(uint32_t fr = 0; fr < m_NumFreqs; fr++)
    {

      list<BellhopResp> results;

      list<BellhopResp> runResults;
      list<TapVec> finalTaps;
      for(uint32_t avg = 0; avg <= m_NumAverages;avg++)
        {
          //Do run
          if(m_NumAverages > 0)
            WriteEnvFile(envFile, m_MinFreqHz + m_FreqStepHz * fr, PerturbSsp());
          else
            WriteEnvFile(envFile, m_MinFreqHz + m_FreqStepHz * fr);
          system(bhcommand.str().c_str());
          ReadArrivals(runResults);
          list<TapVec> run = ProcessTaps(runResults);
          if(avg == 0)
            {
              finalTaps = run;
            }
          else
            {
              NS_LOG_DEBUG("Summing taps: finalTaps.size()=" << finalTaps.size()
                           << " run.size()=" << run.size());
              finalTaps = SumTapVecList(finalTaps, run);
            }

        }
      list<TapVec>::iterator fit = finalTaps.begin();
      list<BellhopResp>::iterator bhrit = runResults.begin();
      UanPdp avgdPdp;
      for(;fit != finalTaps.end();fit++, bhrit++)
        {
          TapVec &a = (*fit);
          double pLoss = 0;
          for(uint32_t i=0; i < a.size(); i++)
            {
              if(m_NumAverages > 0)
                {
                  avgdPdp.AddTap(Tap(Seconds(a[i].first), (a[i].second) / ((double) m_NumAverages)));
                  pLoss += abs(a[i].second) / m_NumAverages;
                }
              else
                {
                  avgdPdp.AddTap(Tap(Seconds(a[i].first), a[i].second));
                  pLoss += abs(a[i].second);
                }
            }
          BellhopResp r(bhrit->GetFreqHz(), bhrit->GetSrcDepthM(),
              bhrit->GetRecDepthM(), bhrit->GetRangeM(), avgdPdp, pLoss);
          results.push_back(r);
        }

#ifdef USE_SQL_DAT
      WriteResults(results, sqldb);
#else
      WriteResults(results);
#endif
    }

}




list<DatMaker::TapVec>
DatMaker::ProcessTaps(list<BellhopResp> &results)
{
  list<BellhopResp>::iterator it = results.begin();
  list<TapVec> out;

  for(;it != results.end();it++)
    {
      //Combine taps
      UanPdp pdp = it->GetPdp();

      TapVec tempTap;
      UanPdp::Iterator pit = pdp.GetBegin();
      for(;pit != pdp.GetEnd();pit++)
        {
          double runTime = 0;
          complex<double> tapSum = 0;
          while(pit != pdp.GetEnd() && runTime < m_resolution)
            {
              runTime += pit->GetDelay().GetSeconds();
              tapSum += pit->GetAmp();
            }
          tempTap.push_back(make_pair(tempTap.size()*m_resolution, tapSum));
        }
      //Find max index
      uint32_t maxIndex = 0;
      double maxValue = -1;
      for(uint32_t i=0;i<tempTap.size();i++)
        {
          if(abs(tempTap[i].second) > maxValue )
            {
              maxIndex = i;
              maxValue = abs(tempTap[i].second);
            }
        }
      TapVec final;
      //Align taps and fill in output TapVec
      uint32_t finalIndex = 0;
      for(;(int32_t) finalIndex < (int32_t) m_PreTaps - (int32_t) maxIndex;finalIndex++)
        {
          final.push_back(make_pair(m_resolution*finalIndex, 0.0));
        }
      uint32_t tempIndex = maxIndex - (m_PreTaps - finalIndex);
      for(;finalIndex < m_TotalTaps && tempIndex < tempTap.size();finalIndex++, tempIndex++)
        {
          final.push_back(make_pair(m_resolution*finalIndex,
                                 tempTap[tempIndex].second));
        }
      for(;finalIndex < m_TotalTaps;finalIndex++)
        {
          final.push_back(make_pair(m_resolution*finalIndex, 0.0));
        }
      out.push_back(final);
    }
  return out;
}
void
DatMaker::ReadArrivals(list<BellhopResp> &results)
{
  ifstream inf("ARRFIL");
  NS_ASSERT(inf.is_open());

  results.clear();

  uint32_t numSrcDepths, numRecDepths, numRanges, numArrivals;
  double freq;

  inf >> freq >> numSrcDepths >> numRecDepths >> numRanges;// >> sd2;
  //inf >> numArrivals;
  NS_LOG_DEBUG("Freq: " << freq << endl
            << "Number src depths: " << numSrcDepths << endl
            << "Number rec depths: " << numRecDepths << endl
            << "Number ranges: " << numRanges << endl);
        //    << "sd2?: " << sd2);

  vector<uint32_t> srcDepths, recDepths;
  vector<double> ranges;
  double depthBuf;
  double rangeBuf;

  for(uint32_t i=0;i<numSrcDepths;i++)
    {
      inf >> depthBuf;
      srcDepths.push_back((uint32_t) depthBuf);
      NS_LOG_DEBUG("Src depth: " << depthBuf);
    }
  for(uint32_t i=0;i<numRecDepths;i++)
    {
      inf >> depthBuf;
      recDepths.push_back((uint32_t) depthBuf);
      NS_LOG_DEBUG("Rec Depth: " << depthBuf);
    }
  for(uint32_t i=0;i<numRanges;i++)
    {
      inf >> rangeBuf;
      ranges.push_back(rangeBuf);
      NS_LOG_DEBUG("Range: " << rangeBuf);
    }

  vector<uint32_t>::iterator srcDepIt, recDepIt;
  vector<double>::iterator rangeIt;


  for(srcDepIt = srcDepths.begin();srcDepIt != srcDepths.end();srcDepIt++)
    {
      inf >> numArrivals;
      NS_LOG_DEBUG("Max arrivals for src depth " << *srcDepIt << "  = " << numArrivals);
      for(recDepIt=recDepths.begin();recDepIt != recDepths.end();recDepIt++)
        for(rangeIt=ranges.begin();rangeIt != ranges.end(); rangeIt++)
          {

            inf >> numArrivals;
            NS_LOG_DEBUG("Looking for " << numArrivals << " arrivals at range " << *rangeIt << " and frequency " << freq << " (NOTE:  These are converted in integers in next line)");

            BhIndex newIndex((uint32_t) freq, *srcDepIt, *recDepIt, (uint32_t) *rangeIt);

            double aMag, aArg, aDelay, aSrcAngle, aRecAngle;
            uint32_t aTopBounces, aBotBounces;
            double pathLoss = 0;
            list<pair<Time, complex<double> > > tapPairs;
            for(uint32_t i=0;i<numArrivals;i++)
              {
                inf >> aMag >> aArg >> aDelay >> aSrcAngle >> aRecAngle >> aTopBounces >> aBotBounces;
                NS_ASSERT(inf.good());
                complex<double> amplitude(aMag*cos(aArg), aMag*sin(aArg));
                pathLoss += abs(amplitude);
                //NS_LOG_DEBUG("Got tap.  Delay = " << aDelay << "   Amp = " << amplitude);
                tapPairs.push_back(make_pair(Seconds(aDelay), amplitude));
              }

            UanPdp newPdp;
            list<pair<Time, complex<double> > >::iterator tapsIt = tapPairs.begin();

            for(;tapsIt != tapPairs.end();tapsIt++)
              {
                newPdp.AddTap(Tap((*tapsIt).first, (*tapsIt).second));
              }
            pathLoss = 20*log10(pathLoss);
            BellhopResp newResp((uint32_t) freq, *srcDepIt, *recDepIt, (uint32_t) *rangeIt, newPdp, pathLoss);

            NS_LOG_DEBUG("Read " << newResp.GetPdp().GetNTaps() << " taps for SD=" << newResp.GetSrcDepthM() << ", RD=" << newResp.GetRecDepthM() << " Range=" << newResp.GetRangeM() << "    Pathloss=" << newResp.GetPathLossDb());
            results.push_back(newResp);

          }
    }
}


#ifdef USE_SQL_DAT
void
DatMaker::WriteResults(list<BellhopResp> &results, sqlite3 *sqldb)
{
  ostringstream sqlStr;
  //expecting results to be in the order read into in ReadArrivals.
  //i.e. sd1 rd1 rr1 .. rrn rd2 rr1 .. rrn .. rdn rr1 rrn sd2 rd1 rr1 ... sdn

  char *err;
  int32_t rc;
  static uint32_t tapid = 0;

  //list<BellhopResp>::iterator it = results.front();
  for (uint32_t sd = 0; sd < m_NumSrcDepth; sd++)
    {
      NS_ASSERT(!results.empty());
      uint32_t srcDepth = results.front().GetSrcDepthM();
      uint32_t freq = results.front().GetFreqHz();

      for (uint32_t rd = 0; rd < m_NumRecDepth; rd++)
        {
          uint32_t recDepth = m_MinRecDepthM + rd*m_RecDepthStepM;

          for (uint32_t rr = 0; rr < m_NumRanges; rr++)
            {
              tapid++;
              BellhopResp resp = results.front();
              results.pop_front();
              NS_LOG_DEBUG("Results source depth: " << resp.GetSrcDepthM() << " Expected depth: " << m_MinSrcDepthM + sd*m_SrcDepthStepM);
              NS_LOG_DEBUG("Results receiver depth: " << resp.GetRecDepthM() << " Expected depth: " << m_MinRecDepthM + rd*m_RecDepthStepM);
              NS_LOG_DEBUG("Results range: " << resp.GetRangeM() << " Expected range: " << 1000*(m_MinRangeKm + rr*m_RangeStepKm));
              NS_ASSERT(resp.GetSrcDepthM() == m_MinSrcDepthM + sd* m_SrcDepthStepM);
              NS_ASSERT(resp.GetRecDepthM() == m_MinRecDepthM + rd*m_RecDepthStepM);
              uint32_t range = 1000*(m_MinRangeKm + rr*m_RangeStepKm);

              UanPdp pdp = resp.GetPdp();
              sqlStr.str("");
              sqlStr << "INSERT INTO indextable (srcdepth, recdepth, range, freq, ploss) VALUES ("
                     << srcDepth << ", " << recDepth << ", " << range << ", " << freq << ", " << resp.GetPathLossDb() << ");";
              rc = sqlite3_exec(sqldb, sqlStr.str().c_str(), &sqlCallback, 0, &err);
              if(rc)
                {
                  sqlite3_close(sqldb);
                  NS_FATAL_ERROR("SQL error inserting index into indextable.\nError: " << err << "\nQuery: " << sqlStr.str());
                }

              UanPdp::Iterator tapit = pdp.GetBegin();
              for(;tapit != pdp.GetEnd();tapit++)
                {
                  sqlStr.str("");
                  sqlStr << "INSERT INTO taptable (tapid, delay, realamp, imagamp) VALUES " <<
                            "(" << tapid << ", " << tapit->GetDelay().GetSeconds() <<
                            ", " << real(tapit->GetAmp()) << ", " << imag(tapit->GetAmp()) << ");";
                  std::cout << sqlStr.str() << endl;
                  rc = sqlite3_exec(sqldb, sqlStr.str().c_str(), &sqlCallback, 0, &err);
                  if(rc)
                    {
                      sqlite3_close(sqldb);
                      NS_FATAL_ERROR("SQL error inserting tap into taptable in.\nError: " << err << "\nQuery: " << sqlStr.str());
                    }

                }
            }
        }

    }
}

sqlite3 *
DatMaker::CreateDb(string dbfile)
{

  char *err;
  int32_t rc;
  ostringstream sqlStr;

  std::ifstream tstfile(dbfile.c_str());
  if(tstfile.is_open())
    {
      NS_FATAL_ERROR("DB file " << dbfile << " already exists!");
    }
  sqlite3 *sqldb;
  rc = sqlite3_open(dbfile.c_str(), &sqldb);
  if(rc)
    {
      sqlite3_close(sqldb);
      NS_FATAL_ERROR("Could not open DB: " << dbfile);
    }
  sqlStr << "CREATE TABLE indextable (tapid INTEGER PRIMARY KEY AUTOINCREMENT, "
         << "srcdepth INTEGER, recdepth INTEGER, range INTEGER, freq INTEGER, ploss DECIMAL);";

  rc = sqlite3_exec(sqldb, sqlStr.str().c_str(), &sqlCallback, 0, &err);
  if(rc)
    {
      sqlite3_close(sqldb);
      NS_FATAL_ERROR("SQL Error creating table, indextable, in " << dbfile << ".\nError: " << err);
    }

  sqlStr.str("");
  sqlStr << "CREATE TABLE taptable (tapid INTEGER, "
         << "delay DECIMAL, realamp DECIMAL, imagamp DECIMAL);";

  rc = sqlite3_exec(sqldb, sqlStr.str().c_str(), &sqlCallback, 0, &err);
  if(rc)
    {
      sqlite3_close(sqldb);
      NS_FATAL_ERROR("SQL Error creating table, taptable, in " << dbfile << ".\nError: " << err);
    }
  return sqldb;
}
#else
{
  void
  DatMaker::WriteResults(list<BellhopResp> &results)
  {
    ostringstream datFileName;
    //expecting results to be in the order read into in ReadArrivals.
    //i.e. sd1 rd1 rr1 .. rrn rd2 rr1 .. rrn .. rdn rr1 rrn sd2 rd1 rr1 ... sdn

    ofstream outf;
    //list<BellhopResp>::iterator it = results.front();
    for (uint32_t sd = 0; sd < m_NumSrcDepth; sd++)
      {
        NS_ASSERT(!results.empty());
        datFileName.str("");
        datFileName << fixed << setprecision(1) << m_DataPath << results.front().GetSrcDepthM() << "-" << results.front().GetFreqHz() << ".dat";
        outf.open(datFileName.str().c_str());
        if(!outf.good())
          {
            NS_FATAL_ERROR("Fatal error opening file: " << datFileName.str());
          }
        for (uint32_t rd = 0; rd < m_NumRecDepth; rd++)
          {
            outf << "rd " << m_MinRecDepthM + rd*m_RecDepthStepM << endl;
            for (uint32_t rr = 0; rr < m_NumRanges; rr++)
              {
                BellhopResp resp = results.front();
                results.pop_front();
                NS_LOG_DEBUG("Results source depth: " << resp.GetSrcDepthM() << " Expected depth: " << m_MinSrcDepthM + sd*m_SrcDepthStepM);
                NS_LOG_DEBUG("Results receiver depth: " << resp.GetRecDepthM() << " Expected depth: " << m_MinRecDepthM + rd*m_RecDepthStepM);
                NS_LOG_DEBUG("Results range: " << resp.GetRangeM() << " Expected range: " << 1000*(m_MinRangeKm + rr*m_RangeStepKm));
                NS_ASSERT(resp.GetSrcDepthM() == m_MinSrcDepthM + sd* m_SrcDepthStepM);
                //NS_ASSERT(resp.GetRangeM() == uint32_t (1000*(m_MinRangeKm+ rr * m_RangeStepKm)));
                NS_ASSERT(resp.GetRecDepthM() == m_MinRecDepthM + rd*m_RecDepthStepM);

                outf << "rr " << resp.GetRangeM() << endl;
                UanPdp pdp = resp.GetPdp();
                outf << resp.GetPathLossDb() << endl;
                outf << pdp;
                outf << endl;

              }
          }
        outf.close();
      }

  }
}
#endif //USQ_SQL_DAT
